/* $Id:  $
 * Functions for controlling a st7565r connected graphical display over
 * SPI. (e.g. EA DOGL128)
 */

#ifndef _LCDST7565R_H_
#define _LCDST7565R_H_

/* Defaults - can be overriden at compile time */
#ifndef LCDPORT
#define LCDPORT PORTA
#define LCDDDR DDRA
#endif
#ifndef LCDA0PIN
#define LCDA0PIN 0
#endif
#ifndef LCDCSPIN
#define LCDCSPIN 1
#endif
#ifndef LCDSCLPIN
#define LCDSCLPIN 2
#endif
#ifndef LCDSIPIN
#define LCDSIPIN 3
#endif

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>

struct lcd_font {
  uint8_t maxwidth;
  uint8_t height;
  prog_uint16_t * offsets;
  prog_uint8_t * data;
};

extern struct lcd_font lcd_defaultfont;

extern uint8_t lcd_displaybuffer[1024];
/* Selects Displaymode. 1 = immediately output, 0 = only output when
 * lcd_updatewholedisplay() is called. */
extern uint8_t lcd_outputmode;

/* Inits the LCD ports and the LCD. */
void lcd_init(void);

/* Forces an update of the whole display. This goes rather fast. */
void lcd_updatewholedisplay(void);

/* Sets an pixel. Depending on lcd_outputmode, this either gets immediately
 * promoted to the display, or the next time lcd_updatewholedisplay() is
 * called. */
void lcd_setpixel(uint8_t x, uint8_t y, uint8_t c);

/* Draws a line */
void lcd_drawline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint8_t c);


#define LCD_IMGFLAG_INVERT      1   /* invert image (white <-> black) */
#define LCD_IMGFLAG_TRANSWHITE  2   /* replace transparency with white */
#define LCD_IMGFLAG_TRANSBLACK  4   /* replace transparency with black */
/* Draws an image. Due to the size of images, it only makes sense to store them
 * in flash, thus there is only a _P function here.
 * Flags (or together what you need): see defines above.
 */
void lcd_drawimage(uint8_t x, uint8_t y, uint8_t flags, PGM_P img);

/* text printing functions */
/* if drawempty is set, the pixels that are not set in the font will be
 * drawn in the background color.
 * c = 0: black text on white background
 * c = 1: white text on black background
 * The return value is the x position after the text was drawn.
 */
/* This draw a single character */
uint8_t lcd_printchar(uint8_t x, uint8_t y, uint8_t drawempty, uint8_t c, uint8_t txt);

/* This draws a string from memory */
uint8_t lcd_printtext(uint8_t x, uint8_t y, uint8_t drawempty, uint8_t c, uint8_t * txt);

/* Same as above, but for strings in program space */
uint8_t lcd_printtext_P(uint8_t x, uint8_t y, uint8_t drawempty, uint8_t c, PGM_P txt);

/* Semi-Internal: Send one byte over the SPI connection to the display. */
void lcd_sendspibyte(uint8_t v);

#endif /* _LCDST7565R_H_ */
