/* $Id: $ */

/* Test for controlling an Electronics Assembly DOG Large graphical Display
 * (128 * 64). */

#include <console.h>
#include <avr/interrupt.h> 
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "eepromdata.h"
#include "lcdst7565r.h"

static prog_uint8_t lcd_image_imgtest1[] = {
    17, 25, /* width, height */
    1, /* flags. 1 = has transparency */
    0x00, 0xff, 0x00, 0xff, 0x00, 0x3f, 0x00, 0xc0, 0x00, 0x3f, 0x00, 0x00, 0x00, 0xbf, 0x00, 0x00, /* binary image data - 8 bytes so far */
    0x00, 0x3d, 0x00, 0xfe, 0x00, 0x71, 0x00, 0xfc, 0x00, 0xe3, 0x00, 0xf8, 0x00, 0xc7, 0x00, 0xf0, /* binary image data - 16 bytes so far */
    0x00, 0x87, 0x00, 0xf1, 0x00, 0x8f, 0x00, 0xe3, 0x00, 0x1f, 0x00, 0xc7, 0x00, 0x3f, 0x00, 0x5e, /* binary image data - 24 bytes so far */
    0x00, 0x80, 0x00, 0x7e, 0x00, 0x00, 0x00, 0x7e, 0x00, 0x01, 0x00, 0x7a, 0x00, 0xfc, 0x00, 0xe3, /* binary image data - 32 bytes so far */
    0x00, 0xf8, 0x00, 0xc7, 0x00, 0xf1, 0x00, 0x8f, 0x00, 0xe1, 0x00, 0x0f, 0x00, 0xe3, 0x00, 0x1f, /* binary image data - 40 bytes so far */
    0x00, 0xc7, 0x00, 0x3f, 0x00, 0x8e, 0x00, 0x7f, 0x00, 0xbc, 0x00, 0x00, 0x00, 0xfd, 0x00, 0x00, /* binary image data - 48 bytes so far */
    0x00, 0xfc, 0x00, 0x03, 0x00, 0xfc, 0x00, 0xff, 0x00, 0xff, 0x00, 0x01
};

static prog_uint8_t lcd_image_imgtest2[] = {
    17, 25, /* width, height */
    1, /* flags. 1 = has transparency */
    0x00, 0xff, 0x00, 0xff, 0xc0, 0x3f, 0x3f, 0xc0, 0xc0, 0x3f, 0xff, 0x00, 0x00, 0xbf, 0xff, 0x00, /* binary image data - 8 bytes so far */
    0x02, 0x3d, 0x00, 0xfe, 0x0e, 0x71, 0x00, 0xfc, 0x1c, 0xe3, 0x00, 0xf8, 0x38, 0xc7, 0x00, 0xf0, /* binary image data - 16 bytes so far */
    0x78, 0x87, 0x00, 0xf1, 0x70, 0x8f, 0x00, 0xe3, 0xe0, 0x1f, 0x00, 0xc7, 0xc0, 0x3f, 0x81, 0x5e, /* binary image data - 24 bytes so far */
    0x7f, 0x80, 0x81, 0x7e, 0xff, 0x00, 0x81, 0x7e, 0xfe, 0x01, 0x81, 0x7a, 0x03, 0xfc, 0x00, 0xe3, /* binary image data - 32 bytes so far */
    0x07, 0xf8, 0x00, 0xc7, 0x0e, 0xf1, 0x00, 0x8f, 0x1e, 0xe1, 0x00, 0x0f, 0x1c, 0xe3, 0x00, 0x1f, /* binary image data - 40 bytes so far */
    0x38, 0xc7, 0x00, 0x3f, 0x70, 0x8e, 0x00, 0x7f, 0x40, 0xbc, 0xff, 0x00, 0x00, 0xfd, 0xff, 0x00, /* binary image data - 48 bytes so far */
    0x03, 0xfc, 0xfc, 0x03, 0x03, 0xfc, 0x00, 0xff, 0x00, 0xff, 0x00, 0x01
};

static prog_uint8_t lcd_image_imgtest3[] = {
    17, 25, /* width, height */
    1, /* flags. 1 = has transparency */
    0x00, 0xff, 0x00, 0xff, 0xc0, 0x3f, 0x3f, 0xc0, 0xc0, 0x3f, 0xff, 0x00, 0x00, 0xbf, 0xff, 0x00, /* binary image data - 8 bytes so far */
    0x02, 0x3d, 0x00, 0xfe, 0x0e, 0x71, 0x00, 0xfc, 0x1c, 0xe3, 0x00, 0xf8, 0x38, 0xc7, 0x00, 0xf0, /* binary image data - 16 bytes so far */
    0x78, 0x87, 0x00, 0xf1, 0x70, 0x8f, 0x00, 0xe3, 0xe0, 0x1f, 0x00, 0xc7, 0xc0, 0x3f, 0x81, 0x5e, /* binary image data - 24 bytes so far */
    0x7f, 0x80, 0x81, 0x7e, 0xff, 0x00, 0x01, 0x7e, 0xfe, 0x01, 0x05, 0x7a, 0x00, 0xfc, 0x1c, 0xe3, /* binary image data - 32 bytes so far */
    0x00, 0xf8, 0x38, 0xc7, 0x00, 0xf1, 0x70, 0x8f, 0x00, 0xe1, 0xf0, 0x0f, 0x00, 0xe3, 0xe0, 0x1f, /* binary image data - 40 bytes so far */
    0x00, 0xc7, 0xc0, 0x3f, 0x01, 0x8e, 0x80, 0x7f, 0x03, 0xbc, 0xff, 0x00, 0x02, 0xfd, 0xff, 0x00, /* binary image data - 48 bytes so far */
    0x03, 0xfc, 0xfc, 0x03, 0x03, 0xfc, 0x00, 0xff, 0x00, 0xff, 0x00, 0x01
};

int main(int argc, char ** argv) {
  /* First try to calibrate the internal oscilator */
  uint8_t tmp8 = eeprom_read_byte(&ee_osccalib);
  if (tmp8 != 0) {
    OSCCAL = tmp8;
  }

  console_init();
  
  /* Try to init LCD */
  lcd_init();
  
  sei();
  
  lcd_drawline(0, 0, 127, 63, 1);
  lcd_drawline(0, 63, 127, 0, 1);
  lcd_drawline(0, 10, 127, 10, 1);
  lcd_drawline(10, 0, 10, 63, 1);
#if 1
  lcd_printtext_P(0, 0, 0, 1, PSTR("This is a test."));
  lcd_printtext_P(0, 16, 1, 1, PSTR("ABCDEFGHIJKLMNOPQRSTUVWXYZ"));
  lcd_printtext_P(0, 32, 1, 0, PSTR("abcdefghijklmnopqrstuvwxyz"));
  lcd_printtext_P(0, 48, 0, 1, PSTR("0123456789?!.,;-"));
#endif
  lcd_updatewholedisplay();
  while (1) {
    if (tmp8) {
      lcd_drawimage(40, 24, 0, lcd_image_imgtest2);
      lcd_drawimage(40+20, 24, 0, lcd_image_imgtest3);
    } else {
      lcd_drawimage(40, 24, 0, lcd_image_imgtest1);
      lcd_drawimage(40+20, 24, 0, lcd_image_imgtest1);
    }
    lcd_updatewholedisplay();
    _delay_ms(500.0); /* Give it time to display */
    tmp8 = !tmp8;
  }
}
